#!/usr/bin/env node
const { Signale } = require('signale');
const CMD = require('commander');
const path = require('path');
const Store = require('./store');
const createLib = require('./index');

const systemLogger = new Signale({

}).scope('system');

const run = async (location, {
  verbose = false,
  storepath,
}) => {
  const realStoreLocation = storepath ? path.resolve(storepath) : undefined;
  const store = new Store(realStoreLocation);
  const logger = new Signale({
    logLevel: verbose ? 'info' : 'warn',
    types: {
      debug: {
        color: 'gray',
        badge: '☢︎'
      },
    },
  });
  const { runSteps, lib } = createLib(logger, store);
  const moduleLocation = path.resolve(location);
  logger.debug(`starting script ${location}`)
  const module = require(moduleLocation);
  await Promise.resolve(module(lib));
  await runSteps();
};

const runAsync = (fn) => (...args) => {
  Promise.resolve(fn(...args))
    .catch((err) => {
      systemLogger.error(err);
    });
};

const command = CMD.command('run <SCRIPT>');
command.option('-v --verbose');
command.option('-s --storepath <storepath>');
command.action(runAsync(run));

CMD.parse(process.argv);
