const request = require('axios');
const nc = require('./nc');
const ssh = require('./ssh');
const createSteps = require('./step');
const sshKey = require('./ssh/key');
const axiosCookieJarSupport = require('axios-cookiejar-support').default;
const tough = require('tough-cookie');
const payloads = require('./payloads');
const http = require('./http');
const { JSDOM } = require("jsdom");

axiosCookieJarSupport(request);

module.exports = (logger, store) => {
  const userLog = logger.scope('👤 user');
  const { runSteps, step } = createSteps(logger);
  return {
    lib: {
      nc: nc(logger),
      ssh: ssh(logger),
      sshKey: sshKey(logger, store),
      payloads: payloads(logger),
      request,
      log: userLog,
      http: http(logger),
      step,
      JSDOM,
      xstep: () => {},
      createCookieJar: () => new tough.CookieJar(),
      store,
      session: () => {
        const instance = request.create({
          jar: new tough.CookieJar(),
        });
        axiosCookieJarSupport(instance);
        instance.defaults.jar = new tough.CookieJar();
        instance.defaults.withCredentials = true;
        return instance;
      }
    },
    runSteps,
  };
};
