const net = require('net');
const EventEmitter = require('eventemitter3');

let sessionId = 1;

module.exports = (signale) => {
  class NCSession extends EventEmitter {
    constructor(port) {
      super();
      this.sessionId = sessionId++;
      this.logger = signale.scope('📶 nc', sessionId.toString());
      this.loggerOut = signale.scope('📶 nc', sessionId.toString());
      this.server = net.createServer(this.onConnect.bind(this));
      this.server.listen(port);
    }

    onConnect(socket) {
      this.socket = socket;
      socket.on('data', (data) => {
        this.emit('data', data);
        if (!this.focus) {
          this.loggerOut.debug('\n' + data.toString());
        }
      });
      this.emit('connected');
      this.logger.debug('connected');
    }

    connection() {
      return new Promise((resolve) => {
        if (this.socket) {
          return resolve(this.socket);
        }
        this.logger.debug('waiting for connection');
        this.once('connected', () => resolve(this.socket));
      });
    }

    end() {
      this.socket.end();
    }

    async write(data) {
      await this.connection();
      this.socket.write(`${data}\n`);
    }

    async addSSHKeys(keys) {
      await this.write(`echo "${keys.publicKey} > ~/.ssh/authorized_keys"`)
    }

    focus() {
      this.focus = true;
      process.stdin.pipe(this.socket);
      this.socket.pipe(process.stdout);
    }

    output() {
      return new Promise(async (resolve) => {
        await this.connection();
        this.socket.once('data', (data) => {
          resolve(data.toString())
        });
      });
    }
  }

  return (port) => new NCSession(port);
};
