const Client = require('ssh2').Client;
const EventEmitter = require('eventemitter3');

let sessionId = 1;

module.exports = (signale) => {
  class SSHSession extends EventEmitter {
    constructor(host, options = {
      username: 'root',
    }) {
      super();
      this.sessionId = sessionId++
      this.logger = signale.scope('ssh', (this.sessionId).toString());
      this.loggerOut = signale.scope('ssh', (this.sessionId).toString(), 'output');
      this.logger.debug(`connecting ${options.username}@${host}`);
      this.session = new Client();
      this.session.on('ready', () => {
        this.connected = true;
        this.emit('connected');
        this.logger.debug('connected');
      });
      this.session.connect({
        host,
        ...options
      });
    }

    connection() {
      return new Promise((resolve) => {
        if (this.connected) {
          return resolve(this);
        }
        this.once('connected', () => {
          resolve(this);
        })
      });
    }

    exec(data) {
      return new Promise(async (resolve, reject) => {
        await this.connection();
        this.logger.debug(`running ${data}`);
        this.session.exec(data, (err, stream) => {
          if (err) return reject(err);
          const output = {
            std: '',
            err: '',
            exitCode: '',
            signal: '',
          };
          stream.on('close', (code, signal) => {
            output.exitCode = code;
            output.signal = signal;
            resolve(output);
          }).on('data', (data) => {
            output.std += data.toString()
            this.logger.Out.debug(data.toString());
          }).stderr.on('data', (data) => {
            output.err += data.toString()
            this.loggerOut.error(data.toString());
          });
        });
      });
    }

    output() {
      return new Promise(async (resolve) => {
        await this.connection();
        this.once('data', (data) => {
          resolve(data.toString());
        });
      });
    }

    forwardPort(localPort, remotePort = localPort, remoteHost = '127.0.0.1') {
      return new Promise(async (resolve, reject) => {
        await this.connection();
        this.session.forwardOut('127.0.0.1', localPort, remoteHost, remotePort, (err, stream) => {
          if (err) {
            return reject(err);
          }
          resolve(stream);
        });
      })
    }

    sftp() {
      return new Promise(async (resolve, reject) => {
        await this.connectionn();
        conn.sftp((err, sftp) => {
          if (err) {
            return reject(err);
          }
          resolve(sftp);
        });
      });
    }

    end() {
      this.session.end();
    }
  }

  return (host, options = {}) => new SSHSession(host, options);
}